/*
 * A process monitor abstraction.
 *
 * Copyright (C) 2022, 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/notification.h>
#include <resource/resource.h>

#include "parent_notification_object_interface.h"



/* A process monitor receiving signals from a task. */

class ProcessMonitor : public ParentNotificationObject, public NotificationSupport,
                       public Resource
{
protected:
  bool _debug;

  /* Resources associated with the created process. */

  l4_cap_idx_t _pager = L4_INVALID_CAP, _mapped_pager = L4_INVALID_CAP,
               _parent = L4_INVALID_CAP, _mapped_parent = L4_INVALID_CAP,
               _task = L4_INVALID_CAP, _mapped_task = L4_INVALID_CAP,
               _thread = L4_INVALID_CAP, _mapped_thread = L4_INVALID_CAP;

  /* Notification support. */

  l4_cap_idx_t _notifier = L4_INVALID_CAP;

public:
  explicit ProcessMonitor(bool debug = false);

  /* Server details. */

  virtual ipc_server_default_config_type config();

  virtual void *interface()
  { return static_cast<ParentNotificationObject *>(this); }

  /* Capability management. */

  virtual void set_pager(l4_cap_idx_t cap, l4_cap_idx_t mapped_cap);
  virtual void set_parent(l4_cap_idx_t cap, l4_cap_idx_t mapped_cap);
  virtual void set_task(l4_cap_idx_t cap, l4_cap_idx_t mapped_cap);
  virtual void set_thread(l4_cap_idx_t cap, l4_cap_idx_t mapped_cap);

  /* Lifecycle management. */

  virtual void pager_ended();

  /* Resource methods. */

  virtual void close();

  /* Parent methods. */

  virtual long signal(unsigned long sig, unsigned long val);

  /* Notification methods. */

  virtual long subscribe(l4_cap_idx_t notifier, notify_flags_t flags);

  virtual long unsubscribe();
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
